-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local im  = ui_imgui

local C = {}

C.name = 'Create Roadblock (Mod)'
C.description = 'Places a police roadblock using two vehicles at the given position.'
C.color = im.ImVec4(0.6, 0.5, 0, 0.8)
C.category = 'repeat_instant'

C.pinSchema = {
  { dir = 'in', type = 'flow', name = 'flow' },
  { dir = 'in', type = 'flow', name = 'reset', impulse = true },
  { dir = 'in', type = 'number', name = 'vehId1', description = 'First vehicle id to use for the roadblock.' },
  { dir = 'in', type = 'number', name = 'vehId2', description = 'Second vehicle id to use for the roadblock.' },
  { dir = 'in', type = 'number', name = 'propId', hidden = true, description = '(Optional) Prop id to place in front of the roadblock.' },
  { dir = 'in', type = 'number', name = 'angle', description = '(Optional) Angle adjustment of each vehicle, in degrees.' },
  { dir = 'in', type = 'number', name = 'gap', description = '(Optional) Length of space between the vehicles.' },
  { dir = 'in', type = 'vec3', name = 'position', description = 'Center position of the roadblock.' },
  { dir = 'in', type = 'quat', name = 'rotation', description = 'Entry rotation of the roadblock.' },

  { dir = 'out', type = 'flow', name = 'flow'}
}

function C:init()
  self:reset()
end

function C:reset()
  self.pinOut.flow.value = false
end

function C:_executionStopped()
  self:reset()
end

function C:work()
  if self.pinIn.reset.value then
    self:reset()
  end
  if self.pinIn.flow.value and not self.pinOut.flow.value then
    if not self.pinIn.vehId1.value or not self.pinIn.vehId2.value or not self.pinIn.position.value or not self.pinIn.rotation.value then
      return
    end

    local veh1 = be:getObjectByID(self.pinIn.vehId1.value)
    local veh2 = be:getObjectByID(self.pinIn.vehId2.value)
    if not veh1 or not veh2 then return end

    local pos = vec3(self.pinIn.position.value)
    local rot = quat(self.pinIn.rotation.value)
    local dirVec = vec3(0, 1, 0):rotated(rot)
    local upVec = vec3(0, 0, 1):rotated(rot)
    local rightVec = dirVec:cross(upVec)
    local angle = self.pinIn.angle.value or 0
    local gap = self.pinIn.gap.value and math.max(0, self.pinIn.gap.value) or 0.2

    for i = -1, 1, 2 do
      local veh = i == 1 and veh1 or veh2

      local quatRight = quatFromDir(rightVec * i, upVec)
      local quatAngle = quatRight * quatFromEuler(0, 0, math.rad(angle * i))
      local vehLength = veh.initialNodePosBB:getExtents().y
      local sideDist = vehLength * 0.5 + gap
      local offset = veh:getInitialNodePosition(veh:getRefNodeId())
      offset = offset:rotated(quatAngle)

      spawn.safeTeleport(veh, pos + (rightVec * sideDist * i) - vec3(offset.x, offset.y, 0), quatAngle, true)
    end

    if self.pinIn.propId.value then
      local prop = be:getObjectByID(self.pinIn.propId.value)
      if prop then
        local vehLength = prop.initialNodePosBB:getExtents().y
        local offset = prop:getInitialNodePosition(prop:getRefNodeId())
        offset = offset:rotated(rot)

        spawn.safeTeleport(prop, pos - (dirVec * (vehLength * 0.5 + 2 + gap)) - vec3(offset.x, offset.y, 0), rot, true)
      end
    end

    self.pinOut.flow.value = true
  end
end

return _flowgraph_createNode(C)
